

function SpokesMinExcErrLocGlobSARPuPow_sms(filepath,display,do_mls,opt_grad)
% function SpokesMinExcErrLocGlobSARPuPow_sms(filepath,display,do_mls,opt_grad)

close all;

% read parameters
disp('Reading pulse parameters ...');
[adj system opt spokes sar]=read_spokes_def_file(filepath);
if display==1
    opt.display='on';
else
    opt.display='off';
end

% adjust main sampling rate to avoid slice aliasing
Gss=system.gmax * spokes.SS_grad_down;
df=abs( 42.576*1e6*Gss*max(abs(adj.zs)) );
dt=1/df/10.0;
if dt<system.deltat
    fprintf('Adjusting system sampling rate from %3.3e sec. to %3.3e sec. to avoid slice aliasing ...\n',system.deltat,dt);
    system.deltat=dt;
end

disp('Preparing optimization ...');
spokes=prepare_pulse(spokes,adj,system,opt.display);
adj=prepare_optimization(opt,spokes,system,adj);
spokes.ntimes*system.deltat

% slice profiles
zmax=( max(abs(adj.zs)) + spokes.sthick )*1.2;
zs=linspace(-zmax,zmax,300)';
SPs=zeros( 300,adj.nz );
SPs_mean=zeros( adj.nz,1 );
for sl=1:adj.nz
    [mxz myz mzz]=bloch_simulation_z(spokes,system,zs,sl);
    SPs(:,sl)=sqrt( abs(mxz).^2 + abs(myz).^2 );
    
    zind=find( abs(zs-adj.zs(sl))<=spokes.sthick/2  );
    SPs_mean(sl)=mean(SPs(zind,sl),1);
end
SPs_max=max(SPs(:));
SPs=SPs / SPs_max;
SPs_mean=SPs_mean / SPs_max;

if display==1
    figure; plot(zs*1e3,SPs,'-','LineWidth',2);
    xlabel('z [mm]');
    title('Slice profiles');
    axis([-zmax*1000 zmax*1000 0 1.15]);
end

% read sar matrices
disp('Reading SAR matrices ...');
% sar=prepare_sar(sar,spokes,opt);

% optimize gradient trajectory (Larmor frequency / LS cost function)
if opt_grad==1
    fprintf('Optimizing gradient trajectory ...\n');
    display_=opt.display;
    opt.display='off';  % force no display during optimization of the spokes locations           
    spokes=opt_gradients(spokes,system,opt,sar,adj,do_mls,SPs_mean);
    opt.display=display_;
    spokes.spcoords
    spokes=prepare_pulse(spokes,adj,system,opt.display);
end

% prepare pulse & compute SM (multiple frequencies)
ncunk=adj.nz * system.ncoils * spokes.nspokes;
[opt.A opt.A_allslices]=compute_system_matrix(adj,system,spokes,0,SPs_mean);

% optimize RF waveforms
options=optimset('Display','iter-detailed','MaxFunEvals',10^5,'TolFun',1e-5,'TolCon',1e-5,'MaxIter',5000,...
    'Algorithm','interior-point','GradObj','on','GradConstr','on');

% ACTIVE-SET OPTIONS -- for comparison only...
% options=optimset('Display','iter-detailed','MaxFunEvals',10^5,'TolFun',1e-6,'TolCon',1e-6,'MaxIter',5000,...
%     'Algorithm','active-set','GradObj','on','GradConstr','on');

% BC mode
if do_mls==-1
    fprintf('COMPUTING BIRDCAGE MODE ...\n');
    rf=[];
    for sl=1:adj.nz
        % pha=[1:system.ncoils]'/system.ncoils*2*pi * 1;  % BC
        pha=angle(squeeze(adj.b1maps(:,end/2,end/2,sl)));  % pseudo-CP
        rf=[ rf;exp(1j*pha) ];
    end
    MT=opt.A*rf;
    voltage=mean(abs(adj.mtttot))/mean(abs(MT));
    rf=rf*voltage;  % set RF voltage so as to reach FA
    
    c=fn( [real(rf);imag(rf)],adj,spokes,sar,system );  % degrade so as not to exceed RF constraints
    cmax=max(c);
    if cmax>0
        alpha=sqrt( system.umax^2/( system.umax^2+cmax ) );
        rf=rf*alpha;
    end
    xrf=[ real(rf);imag(rf) ];

elseif do_mls==0 || do_mls==1
    
    if do_mls==0
        fprintf('******** COMPUTING LEAST-SQUARES SOLUTION ********\n');
    else
        fprintf('******** COMPUTING MLS SOLUTION (full optimization) ********\n');
    end
    % LS
    % test_gradient( @(xrf)f0_MLS(xrf,opt.A,adj),xrf0 );
    % test_gradient2( @(xrf)fn(xrf,adj,spokes,sar,system),xrf0 );
    % test_hessian( @(xrf)fn(xrf,adj,spokes,sar,system),@(xrf)dualhess(xrf,lambda,opt.A,adj,spokes,sar,2),xrf0 );
    % test_hessian( @(xrf)f0(xrf,opt.A,adj),@(xrf)dualhess(xrf,lambda,opt.A,adj,spokes,sar,2),xrf0 );
    options.Hessian='user-supplied';
    options.HessFcn=@(xrf,lambda)dualhess(xrf,lambda,opt.A,adj,spokes,sar,1);
    xrf0=zeros(2*ncunk,1);
    [xrf fval exitflag output lambda]=fmincon(@(xrf)f0(xrf,opt.A,adj),xrf0,[],[],[],[],[],[],@(xrf)fn(xrf,adj,spokes,sar,system),options);    

    % MLS refinement
    % lambda=ones(10000,1);
    % test_hessian( @(xrf)f0_MLS(xrf,opt.A,adj),@(xrf)dualhess_MLS(xrf,lambda,opt.A,adj,spokes,sar,2),xrf );
    % test_hessian( @(xrf)fn(xrf,adj,spokes,sar,system),@(xrf)dualhess_MLS(xrf,lambda,opt.A,adj,spokes,sar,2),xrf );
    if do_mls==1
        options.Hessian='user-supplied';
        options.HessFcn=@(xrf,lambda)dualhess_MLS(xrf,lambda,opt.A,adj,spokes,sar,1);    
        % options.Hessian='';
        [xrf fval exitflag output lambda]=fmincon(@(xrf)f0_MLS(xrf,opt.A,adj),xrf,[],[],[],[],[],[],@(xrf)fn(xrf,adj,spokes,sar,system),options);
    end

elseif do_mls==2
    fprintf('******** COMPUTING MLS SOLUTION (phase adoption) ********\n');
    xrf=design_pulse_mls(opt,spokes,sar,adj,system,options);
end
    
rf=xrf(1:ncunk,1) + 1j*xrf(ncunk+1:2*ncunk,1);
rf_sum=sum( reshape(rf.',system.ncoils*spokes.nspokes,adj.nz).',1 );

% process results
display_optimization_summary(xrf,rf,rf_sum,opt,spokes,sar,adj,system);
[rf_exp rf_exp_sum]=write_and_display_pulse(rf,opt,spokes,system,adj);
MT_STA=write_and_display_magnetization(rf,adj,system,spokes,SPs_mean,opt);

MT_Bloch=cell(adj.nz,1);
for i=1:adj.nz
    [mx my mz]=bloch_simulation(rf_exp{i},adj,spokes,system,i);
    MT_Bloch{i,1}=( mx+1j*my )*SPs_mean(i);
end
write_and_display_bloch_simulation(MT_Bloch,opt,adj);

g=spokes.g;
k=spokes.k;
SPs=[zs SPs];
save pulse_sms.mat rf rf_exp rf_exp_sum g k MT_STA MT_Bloch SPs;


















