


function [mxf myf mzf]=bloch_simulation(rf,adj,spokes,system,slice)
% function [mxf myf mzf]=bloch_simulation(rf,adj,spokes,system,slice)

mxy0=0.0; mz0=1.0;  % initial magnetization state
gamma=42.576*1e6;

mxf=zeros(adj.nx,adj.ny);
myf=zeros(adj.nx,adj.ny);
mzf=zeros(adj.nx,adj.ny);
q=zeros(adj.slices{slice}.nnonzeropixels,4); q(:,1)=1; q(:,4)=1;
b1maps=zeros(system.ncoils,adj.slices{slice}.nnonzeropixels);
for chn=1:system.ncoils
    b1map=squeeze( adj.b1maps(chn,:,:,slice) );
    b1maps(chn,:)=b1map(adj.slices{slice}.ind);
end
coords=adj.slices{slice}.nonzeropixels(:,4:6);

gradients=spokes.g;
% gradients(:,3)=-gradients(:,3);

ntimes=spokes.ntimes;
% ntimes=ceil( 12e-3/system.deltat );

fprintf('Bloch simulation of slice %d/%d ',slice,adj.nz);
sbuf=[];
for time=1:ntimes
    
    if mod(time,10)==0
        for j=1:size(sbuf,2)
            fprintf('\b');
        end
        sbuf=sprintf('[%d/%d]',time,ntimes);
        fprintf(sbuf);
    end

    % Cayley-Klein parameters
    if time<=spokes.ntimes
        b1s=( rf(time,:)*b1maps ).';
        grad=coords*gradients(time,:)' + adj.slices{slice}.b0_/gamma;
    else
        b1s=zeros(size(b1s));
        grad=b0/gamma;
    end        
    norm=sqrt( abs(b1s).^2+grad.^2 );
    phi=-2*pi*gamma*system.deltat*norm;
    if sum(abs(phi))<=0.0
        continue;
    end
    n=[real(b1s) imag(b1s) grad];
    n=n./repmat(norm,1,3);
    a=cos(phi/2.0) - 1j*(n(:,3)).*sin(phi/2.0);
    b=sin(phi/2.0).*n(:,2) - 1j*sin(phi/2.0).*n(:,1);    
    ind2=find(norm==0 | isnan(norm));
    a(ind2)=1.0;
    b(ind2)=0.0;
    
    % update rotation matrix
    q1=q(:,1); q2=q(:,2); q3=q(:,3); q4=q(:,4);
    q(:,1)=a.*q1 - conj(b).*q3;
    q(:,2)=a.*q2 - conj(b).*q4;
    q(:,3)=b.*q1 + conj(a).*q3;
    q(:,4)=b.*q2 + conj(a).*q4;
end

fprintf('\n');

% final magnetization
a=q(:,1); b=q(:,3);
mxy=(conj(a).*conj(a))*mxy0 - (b.*b).*conj(mxy0) + (2.0*conj(a).*b).*mz0;
mz=(-conj(a).*conj(b))*mxy0 - (a.*b).*conj(mxy0) + (a.*conj(a)-b.*conj(b))*mz0;

mxf(adj.slices{slice}.ind)=real(mxy);
myf(adj.slices{slice}.ind)=imag(mxy);
mzf(adj.slices{slice}.ind)=real(mz);



