

function tmax=compute_step_max(x0,dir,tmax_ini,opt,spokes,sar)
% function tmax=compute_step_max(x0,dir,tmax_ini,opt,spokes,sar)

tmax=tmax_ini;

% SAR constraints
nchannels=opt.nrows*opt.ncoilspr;
ncunk=size(x0,1)/2;

rf_x0=reshape( x0(1:ncunk) + 1j*x0(ncunk+1:2*ncunk),1,ncunk );  % row vector
rf2_x0=reshape(rf_x0.',nchannels*spokes.nspokes,opt.nslices).';  % nslices*(nspokes*nchannels) matrix with the RF pulses for all pulses

rf_dir=reshape( dir(1:ncunk) + 1j*dir(ncunk+1:2*ncunk),1,ncunk );  % row vector
rf2_dir=reshape(rf_dir.',nchannels*spokes.nspokes,opt.nslices).';  % nslices*(nspokes*nchannels) matrix with the RF pulses for all pulses

% solve a second order equation for each VOP
for i=1:sar.nvop+1    
    A=0;
    B=0;
    C=0;
    for sp=1:spokes.nspokes
        inds=(sp-1)*nchannels+1:sp*nchannels;
        tmp1=[];
        tmp2=[];
        for sl=1:opt.nslices
            tmp1=[tmp1 reshape(sar.sarmats(i,:,:),nchannels,nchannels)*rf2_x0(sl,inds)'];
            tmp2=[tmp2 reshape(sar.sarmats(i,:,:),nchannels,nchannels)*rf2_dir(sl,inds)'];
        end
        
        for sl=1:opt.nslices
            C=C + real( rf2_x0(sl,inds)*tmp1(:,sl)*spokes.sumsincsq );  % direct term in rf_x0
            A=A + real( rf2_dir(sl,inds)*tmp2(:,sl)*spokes.sumsincsq );  % direct term in rf_dir
            for sl2=1:opt.nslices
                if sl2>sl
                    C=C + 2.0*real( rf2_x0(sl,inds)*tmp1(:,sl2)*( spokes.sinc(:,sl2,sp)'*spokes.sinc(:,sl,sp) ) );  % direct term in rf_x0
                    A=A + 2.0*real( rf2_dir(sl,inds)*tmp2(:,sl2)*( spokes.sinc(:,sl2,sp)'*spokes.sinc(:,sl,sp) ) );  % direct term in rf_dir
                end
                B=B + rf2_dir(sl,inds)*tmp1(:,sl2)*( spokes.sinc(:,sl2,sp)'*spokes.sinc(:,sl,sp) );  % cross term in rf_x0 and rf_rf_dir
            end            
        end
    end
    
    if i==sar.nvop+1
        C=C - sar.gsarmax;
    else
        C=C - sar.lsarmax;
    end    
    B=2*real(B);
    
    delta=B^2 - 4*A*C;
    if delta<0
        error(sprintf('Second order equation for SAR constraint %d does not have solutions.',i));
    end
    sol_max=(-B+sqrt(delta))/(2*A);
    tmax=min(tmax,sol_max);    
end























