

function xrf=design_pulse_mls(opt,spokes,sar,adj,system,options)
% function xrf=design_pulse_mls(opt,spokes,sar,adj,system,options)

tol_mls=1e-3;
niters=10;

global pha;
global phaimg;

ncunk=size(opt.A,2);
iter=0;
phaimg=zeros(adj.nz,adj.nx,adj.ny);  % careful, z is the first dimension!!
smfilter=fspecial('gaussian',[10 10],3);  % 5 pixels Gaussian filter with 1 pixel s.d.

xx0=zeros(2*ncunk,1);
update_target_phase(opt.A,ncunk,adj,xx0,smfilter);
adj.mtttot=abs(adj.mtttot).*exp(1j*pha);

if strcmpi(opt.display,'on')
    f=figure;
    to_disp=conv_3Dstack_2_2Dimg(permute(phaimg,[2 3 1]),1,adj.nz);
    imagesc(to_disp); colormap(hot); colorbar; axis off; axis image; caxis([-pi pi]);
    title(sprintf('Target phase (MLS it. #%d)',iter));
end

% MLS
f1=-1;
f2=-1;
options.Hessian='user-supplied';
options.HessFcn=@(xrf,lambda)dualhess(xrf,lambda,opt.A,adj,spokes,sar,1);
options.Display='none';

while iter<niters || f1<0 || f2<0
    
    iter=iter+1;
    % solve LS with current phase
    xrf0=zeros(2*ncunk,1);
    [xrf f2 exitflag output lambda]=fmincon(@(xrf)f0(xrf,opt.A,adj),xrf0,[],[],[],[],[],[],@(xrf)fn(xrf,adj,spokes,sar,system),options);    

    % iterations statement
    fprintf('\tMLS it. #%d: f=%f\n',iter,f2);
    
    % stopping criterion
    if f1>=0 && f2>=0 && ( abs(f1-f2)<(abs(f1)+abs(f2))*tol_mls || (f2>f1 && iter>1) )
        break;
    end
    
    % update target magnetization phase
    update_target_phase(opt.A,ncunk,adj,xrf,smfilter);
    adj.mtttot=abs(adj.mtttot).*exp(1j*pha);
    
    % display
    if strcmpi(opt.display,'on')
        f=figure;
        to_disp=conv_3Dstack_2_2Dimg(permute(phaimg,[2 3 1]),1,adj.nz);
        imagesc(to_disp); colormap(hot); colorbar; axis off; axis image; caxis([-pi pi]);
        title(sprintf('Target phase (MLS it. #%d)',iter));
    end
    
    f1=f2;
    
end










 

function update_target_phase(A,ncunk,adj,xx,smfilter)

global pha;
global phaimg;

x=xx(1:ncunk) + 1j*xx(ncunk+1:2*ncunk);
y=A*x;
pha=angle(y);

nend=0;
for i=1:adj.nz
    nstart=nend+1;
    nend=nend + adj.slices{i}.nnonzeropixels;
    phaimg(i,adj.slices{i}.ind)=pha(nstart:nend,1);
end





