

function spokes2=prepare_pulse(spokes,adj,system,display)
% function spokes2=prepare_pulse(spokes,adj,system,display)

gamma=42.576*1e6;

spokes2=prepare_gradient_spokes(spokes,system);

N=spokes2.ntimesperspoke;
g=spokes2.g(1:N,:)*spokes2.sdir(:);
spokes2.Gss=abs(g(ceil(N/2),1));  % slice selection plateau
omega=gamma*(spokes2.Gss)*(spokes2.sthick);

times=([1:N]'-0.5-N/2)*system.deltat;
ind_rf=find(spokes2.mask_rf(1:N)>0);
Nrf=ind_rf(end)-ind_rf(1);
Trf=Nrf*system.deltat;

x=omega*times;        

f1=sinc(x);
f2=rect( times/Trf );    
f3=0.54 + 0.46*cos( 2*pi*times/Trf );  % hanning window
        
% basic sinc shape
spokes2.subpulse=f1.*f2.*f3;    
spokes2.subpulse=spokes2.subpulse / max( abs(spokes2.subpulse) );  % normalization of the RF amplitude
spokes2.sumsincsq=sum( abs(spokes2.subpulse).^2 );

% freq modulated sinc shapes
spokes2.sinc=zeros(spokes2.ntimesperspoke,adj.nz);
kz=spokes2.k(1:spokes2.ntimesperspoke,3);
for sl=1:adj.nz  % phase offsets for slice selections
    phaseoff=-2*pi * kz * adj.zs(sl);
    spokes2.sinc(:,sl)=spokes2.subpulse .* exp(1j*phaseoff);
end


if strcmpi(display,'on')
    figure;
    times=[1:spokes2.ntimes]'*system.deltat;
    subplot(2,1,1); plot(times*1e3,spokes2.g(:,1)*1e3,'Color','blue','LineWidth',2); hold on;
    plot(times*1e3,spokes2.g(:,2)*1e3,'Color','red','LineWidth',2);
    plot(times*1e3,spokes2.g(:,3)*1e3,'Color','green','LineWidth',2);
    legend('Gx','Gy','Gz','Location','EastOutside'); legend('boxoff');
    title('Gradient trajectory [mT/m]','FontSize',12);

    subplot(2,1,2); plot(times*1e3,spokes2.k(:,1),'Color','blue','LineWidth',2); hold on;
    plot(times*1e3,spokes2.k(:,2),'Color','red','LineWidth',2);
    plot(times*1e3,spokes2.k(:,3),'Color','green','LineWidth',2);
    legend('kx','ky','kz','Location','EastOutside'); legend('boxoff');
    title('K-space trajectory [1/m]','FontSize',12); xlabel('Time [ms]');
    
    for i=1:adj.nz
        f=figure;
        plot(times(1:N,1)*1e3,abs(spokes2.sinc(:,i)),'LineWidth',2);
        title(sprintf('Slice %d sub-pulses [a.u.]',i)); xlabel('Time [ms]');
    end
    
end






