

function [rf rfsum]=write_and_display_pulse(x,opt,spokes,system,adj)
% function [rf rfsum]=write_and_display_pulse(x,opt,spokes,system,adj)

rf=cell(adj.nz,1);
rfsum=zeros(spokes.ntimes,system.ncoils);

% INI file
options.Oversampling=1;
options.NominalFlipAngle=opt.tfa;
options.RFPULSE_ID=0;
options.VERBOSE=0;
options.GRADRASTERTIME=system.deltat*1e6;  % us
grad=[spokes.g(:,2) -spokes.g(:,1) -spokes.g(:,3)]*1e3;  % swap gx and gy and convert T/m -> mT/m (the delayed version of the gradienst is sent to the scanner)

% save ini file
grad_over_samp_fac=1e-5/system.deltat;
if ceil(grad_over_samp_fac)~=grad_over_samp_fac
    warning('The oversampling factor is not integer, this will cause the ini pulse to be unplayable on the scanner.');
else
    options.GRADRASTERTIME=system.deltat*grad_over_samp_fac*1e6;  % 10 us
    options.FACTOROVERSAMPLE=grad_over_samp_fac;
    grad=grad(1:grad_over_samp_fac:end,:);
end
options.type= 'SBBExcitationPtx';
options.PulseName= 'pTx';
options.Comment = 'pTx pulse';

% individual slices
for sl=1:adj.nz
    file=fopen( sprintf('%s/pulse_slice%d.txt',opt.resdir,sl) ,'w');
%     for j=1:spokes.nspokes
%         fprintf(file,'spoke #%d mag.\tspoke #%d pha.\t',j,j);
%     end
%     for i=1:system.ncoils
%         fprintf(file,'\n');
%         for j=1:spokes.nspokes
%             ind=i + (j-1)*system.ncoils + (sl-1)*system.ncoils*spokes.nspokes;
%             fprintf(file,'%e\t%e\t',abs(x(ind)),angle(x(ind))/pi*180.0);
%         end
%     end
    
%     fprintf(file,'\ntime (s)\tgx (T/m)\tgy (T/m)\tgz (T/m)\t\tsx (T/m/s)\tsy (T/m/s)\tsz (T/m/s)\tkx (1/m)\tky (1/m)\tkz (1/m)');
%     for j=1:system.ncoils
%         fprintf(file,'\trf_%d_mag (V)\trf_%d_pha (deg.)',j,j);
%     end
    time=[1:spokes.ntimes]'*system.deltat;
    rf{sl,1}=zeros(spokes.ntimes,system.ncoils);
    for i=1:spokes.ntimes
        %fprintf(file,'\n');
        if i==1
            sr=[0 0 0];
        else
            sr=(spokes.g(i,:)-spokes.g(i-1,:))/system.deltat;
        end
        %fprintf(file,'%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e',time(i),spokes.g(i,:),sr,spokes.k(i,:));
        for j=1:system.ncoils
            rf{sl,1}(i,j)=get_rf( x((sl-1)*system.ncoils*spokes.nspokes+1:sl*system.ncoils*spokes.nspokes,1),i,j,system.ncoils,spokes,sl );
            %fprintf(file,'\t%e\t%e',abs(rf{sl,1}(i,j)),angle(rf{sl,1}(i,j))/pi*180.0);
        end
    end
    fclose(file);
    
    % ini file
	% write_pulse_ini_file(rf{sl,1},sprintf('%s/pulse_slice%d.ini',opt.resdir,sl),spokes,opt,system);
    
    % update sum of all slices
    rfsum=rfsum+rf{sl,1};
    
    % INI file (SMS)
    options.fileName=sprintf('pTXArbitrary_slice%d',sl);    
    create_pTXRFPulse2( grad.',rf{sl,1}', options);  % save the conjugate of RF pulse

    
%     if strcmpi(opt.display,'on')
%         figure;
%         subplot(2,1,1); plot(time*1e3,abs(rf{sl,1})); xlabel('time [ms]'); title( sprintf('RF voltage amplitudes (slice #%d out of %d) [V]',sl,adj.nz) );
%         subplot(2,1,2); plot(time*1e3,angle(rf{sl,1})/pi*180.0); xlabel('time [ms]'); title( sprintf('RF voltage phases (slice #%d out of %d) [deg.]',sl,adj.nz) );
%     end
end

% sum of all slices
file=fopen( sprintf('%s/pulse_allslices.txt',opt.resdir) ,'w');
fprintf(file,'\ntime (s)\tgx (T/m)\tgy (T/m)\tgz (T/m)\t\tsx (T/m/s)\tsy (T/m/s)\tsz (T/m/s)\tkx (1/m)\tky (1/m)\tkz (1/m)');
% for j=1:system.ncoils
%     fprintf(file,'\trf_%d_mag (V)\trf_%d_pha (deg.)',j,j);
% end
for i=1:spokes.ntimes
    fprintf(file,'\n');
    if i==1
        sr=[0 0 0];
    else
        sr=(spokes.g(i,:)-spokes.g(i-1,:))/system.deltat;
    end
    fprintf(file,'%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e',time(i),spokes.g(i,:),sr,spokes.k(i,:));
    for j=1:system.ncoils
        fprintf(file,'\t%e\t%e',abs(rfsum(i,j)),angle(rfsum(i,j))/pi*180.0);
    end
end
fclose(file);

if strcmpi(opt.display,'on')
    figure;
    subplot(2,1,1); plot(time*1e3,abs(rfsum)); xlabel('time [ms]'); title('SMS pulse [V]');
    subplot(2,1,2); plot(time*1e3,angle(rfsum)/pi*180.0); xlabel('time [ms]'); title('SMS pulse [deg.]');
end

% INI file (SMS)
options.fileName='pTXArbitrary';
create_pTXRFPulse2( grad.',rfsum', options);  % save the conjugate of RF pulse


    
    
function rf=get_rf(x,time,channel,nchannels,spokes,slice)

rf=0.0;
if time>0 && time<=spokes.ntimes && spokes.time_to_spoke(time)>0 && channel>0 && channel<=nchannels
    rf=x(channel + (spokes.time_to_spoke(time)-1)*nchannels)*spokes.sinc(spokes.time_to_sinc_time(time),slice);
end





function write_pulse_ini_file(rf,filepath,spokes,opt,system)

file=fopen(filepath,'w');
system.ncoils=size(rf,2);
ntimes=size(rf,1);

% global parameter section
%fprintf(file,'\n#----------\n');
%fprintf(file,'[pTXPulse]\n');
%fprintf(file,'\n');
%fprintf(file,'NUsedChannels    = %d\n',system.ncoils);
%fprintf(file,'DimRF            = 2\n');
%fprintf(file,'DimGradient      = 3\n');
%fprintf(file,'MaxAbsRF         = %g\t\t # scaling for RF amplitude\n',max(max(abs(rf))));
%fprintf(file,'InitialPhase     = 0.0\n');	
%fprintf(file,'Asymmetry        = 0.5\n');
%fprintf(file,'\n');
%fprintf(file,'PulseName        = %s\t\t #standard RF pulse parameters\n',filepath);
%fprintf(file,'Family           = pTX\n');
%fprintf(file,'Comment          = pTX\n');
%fprintf(file,'NominalFlipAngle = %g\n',opt.tfa);
%fprintf(file,'Samples          = %d\n',ntimes);
%fprintf(file,'DwellTime          = %e\n',system.deltat);
%fprintf(file,'AmplInt          = 100.0\n');
%fprintf(file,'AbsInt           = 100.0\n');
%fprintf(file,'PowerInt         = 100.0\n');
%fprintf(file,'MinSlice         = 1.0\n');
%fprintf(file,'MaxSlice         = 1.0\n');
%fprintf(file,'RefGrad          = 1.0\n');
%fprintf(file,'\n');

% SAR section
%fprintf(file,'\n#----------\n');
%fprintf(file,'[TXA_SAR_SECTION]\n');
%fprintf(file,'\n');
%fprintf(file,'IsPulseSARchecked = 0\n');

%fprintf(file,'TX1_PALI_LIMIT_PEAK=\"0\"\n');
%fprintf(file,'TX1_PALI_LIMIT_10SEC=\"0\"\n');
%fprintf(file,'TX1_PALI_LIMIT_6MIN=\"0\"\n\n');

%fprintf(file,'TX2_PALI_LIMIT_PEAK=\"0\"\n');
%fprintf(file,'TX2_PALI_LIMIT_10SEC=\"0\"\n');
%fprintf(file,'TX2_PALI_LIMIT_6MIN=\"0\"\n\n');

%fprintf(file,'TX3_PALI_LIMIT_PEAK=\"0\"\n');
%fprintf(file,'TX3_PALI_LIMIT_10SEC=\"0\"\n');
%fprintf(file,'TX3_PALI_LIMIT_6MIN=\"0\"\n\n');

%fprintf(file,'TX4_PALI_LIMIT_PEAK=\"0\"\n');
%fprintf(file,'TX4_PALI_LIMIT_10SEC=\"0\"\n');
%fprintf(file,'TX4_PALI_LIMIT_6MIN=\"0\"\n\n');

%fprintf(file,'TX5_PALI_LIMIT_PEAK=\"0\"\n');
%fprintf(file,'TX5_PALI_LIMIT_10SEC=\"0\"\n');	
%fprintf(file,'TX5_PALI_LIMIT_6MIN=\"0\"\n\n');

%fprintf(file,'TX6_PALI_LIMIT_PEAK=\"0\"\n');
%fprintf(file,'TX6_PALI_LIMIT_10SEC=\"0\"\n');
%fprintf(file,'TX6_PALI_LIMIT_6MIN=\"0\"\n\n');

%fprintf(file,'TX7_PALI_LIMIT_PEAK=\"0\"\n');
%fprintf(file,'TX7_PALI_LIMIT_10SEC=\"0\"\n');
%fprintf(file,'TX7_PALI_LIMIT_6MIN=\"0\"\n\n');

%fprintf(file,'TX8_PALI_LIMIT_PEAK=\"0\"\n');
%fprintf(file,'TX8_PALI_LIMIT_10SEC=\"0\"\n');
%fprintf(file,'TX8_PALI_LIMIT_6MIN=\"0\"\n\n');

% gradient section
%fprintf(file,'\n#----------\n');
%fprintf(file,'[Gradient]\t # Gx\t Gy\t Gz\t [mT/m]\n');
%fprintf(file,'\n');
%fprintf(file,'GradientSamples   =  %d\n',ntimes);
%fprintf(file,'MaxAbsGradient[0] =  %f\t %f\t %f\t\t # scaling for G amplitude\n',max(abs(spokes.g(:,1)))*1e3,max(abs(spokes.g(:,2)))*1e3,max(abs(spokes.g(:,3)))*1e3);  % T/m -> mT/m
%fprintf(file,'GradRasterTime    =  %f   # [us]\n',system.deltat*1e6);  %  s -> us
%fprintf(file,'\n');

for i=1:ntimes
    %fprintf(file,'G[%d]= %f\t %f\t %f\n',i-1,spokes.g(i,1)*1e3,spokes.g(i,2)*1e3,spokes.g(i,3)*1e3);  %  T/m -> mT/m
end

% rf section
for j=1:system.ncoils
    %fprintf(file,'\n#----------\n');
    %fprintf(file,'[pTXPulse_ch%d]\t # Amplitude\t Phase\n',j-1);
    %fprintf(file,'\n');
    for i=1:ntimes
        mag=abs(rf(i,j));
        pha=angle(rf(i,j));
        if pha<0 
            pha=pha+2*pi;
        end
        %fprintf(file,'RF[%d]= %f\t %f\n',i-1,mag,pha);
    end
end
%fprintf(file,'\n#EOF\n');
fclose(file);







    
    
    
    
    
    
    
    
    
    


    
    
    
    
    
    
    
    
    
    
    
    
